/**
 * @file highcharttable.js
 * 
 * Takes parameters set on the HighchartTable configuration page and invokes
 * HighchartTable chart-from-table function.
 *
 * @todo add I18n and refactor this rather messy code
 */
(function ($) {

  Drupal.behaviors.highcharttable = {

    attach: function(context, settings) {

      $(settings.highcharttable, context).each(function() {

        // settings.highcharttable is an array of params indexed by selectors as
        // entered on the HighchartTable config page. 1 selector per decoration.
        // In addition global config params are kept under 'global'.
        var basePath = '';
        var linkStyle = null;
        $.each(this, function(selector, params) {

          if (selector === 'global') {
            basePath = params['base-path'] || '';
            linkStyle = params['contextual-links'];
            if (linkStyle !== 'off') {
              var linkHtml = Drupal.behaviors.highcharttable.insertContextLinkHtml(basePath, linkStyle);
              $(linkHtml).insertBefore('table');
            }
          }
          else {
            // table selector
            var decoratedTables = $(selector);
            if (decoratedTables.length > 0) {
              // Set all params on the special "data-graph-..." attributes.
              for (var dataKey in params) {
                if (dataKey !== 'hide-table' && params[dataKey] !== null) {
                  decoratedTables.attr('data-graph-' + dataKey, params[dataKey]);
                }
              }
              // Remove insert link before adding other links and the chart.
              decoratedTables.prev('.highcharttable-region').remove();

              if (linkStyle !== 'off') {
                var linkHtml = Drupal.behaviors.highcharttable.configureDeleteContextLinksHtml(basePath, linkStyle);
                $(linkHtml).insertBefore(decoratedTables);
              }

              decoratedTables.highchartTable();

              if (params['hide-table']) {
                decoratedTables.hide();
              }
            }
          }
        });
      });
    },

    insertContextLinkHtml: function(basePath, linkStyle) {
      var thisPage = window.location.pathname.substr(basePath.length);

      if (linkStyle === 'plain') {
        return '<div class="highcharttable-region">' +
              '<ul>' +
                '<li><a class="highcharttable-insert" href="' + basePath + 'highcharttable/contextual-link/insert?destination=' + thisPage + '">Insert chart from table</a></li>' +
              '</ul>' +
            '</div>';
      }
      return '<div class="highcharttable-region contextual-links-region">' +
          '<div class="contextual-links-wrapper">' +
            '<ul class="contextual-links">' +
              '<li><a class="highcharttable-insert" href="' + basePath + 'highcharttable/contextual-link/insert?destination=' + thisPage + '">Insert chart from table</a></li>' +
            '</ul>' +
          '</div>' +
        '</div>';
    },

    configureDeleteContextLinksHtml: function(basePath, linkStyle) {
      var thisPage = window.location.pathname.substr(basePath.length);

      if (linkStyle === 'plain') {
        return '<div class="highcharttable-region">' +
              '<ul>' +
                '<li><a class="highcharttable-configure" href="' + basePath + 'admin/config/content/highcharttable?destination=' + thisPage + '">Configure this chart</a></li>' +
                '<li><a class="highcharttable-delete" href="' + basePath + 'highcharttable/contextual-link/delete?destination=' + thisPage + '">Delete chart(s) from page</a></li>' +
              '</ul>' +
            '</div>';
      }
      return '<div class="highcharttable-region contextual-links-region">' +
          '<div class="contextual-links-wrapper">' +
            '<ul class="contextual-links">' +
              '<li><a class="highcharttable-configure" href="' + basePath + 'admin/config/content/highcharttable?destination=' + thisPage + '">Configure this chart</a></li>' +
              '<li><a class="highcharttable-delete" href="' + basePath + 'highcharttable/contextual-link/delete?destination=' + thisPage + '">Delete chart(s) from page</a></li>' +
            '</ul>' +
          '</div>' +
        '</div>';
    },
  };

}) (jQuery);

/**
 * Highcharts.numberformat(value, decimals, decPoint, thousandsSep)
 * decPoint (e.g., '.') and thousandsSep (',') are taken from lang/locale.
 */

highcharttableDollarFormatter = function (value) {
  return '$ ' + Highcharts.numberFormat(value, -1);
}

highcharttableEuroFormatter = function (value) {
  return '€ ' + Highcharts.numberFormat(value, -1);
}

highcharttablePoundFormatter = function (value) {
  return '£ ' + Highcharts.numberFormat(value, -1);
}

highcharttableNumberFormatter1Decimal = function (value) {
  return Highcharts.numberFormat(value, 1);
}

highcharttableNumberFormatter2Decimals = function (value) {
  return Highcharts.numberFormat(value, 2);
}

highcharttableNumberFormatter3Decimals = function (value) {
  return Highcharts.numberFormat(value, 3);
}

highcharttableDollarFormatter2Decimals = function (value) {
  return '$ ' + Highcharts.numberFormat(value, 2);
}

highcharttableEuroFormatter2Decimals = function (value) {
  return '€ ' + Highcharts.numberFormat(value, 2);
}

highcharttablePoundFormatter2Decimals = function (value) {
  return '£ ' + Highcharts.numberFormat(value, 2);
}

highcharttablePercentFormatter = function (value) {
  return value + ' %';
}
